/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *     "Apache Jetspeed" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" or
 *    "Apache Jetspeed", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jetspeed.portal.portlets.util.poll;

import org.apache.turbine.om.*;


/**
Provides an Answer for a question.

@author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
@version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
*/
public class Answer extends BaseObject {
    
    private String title = "";
    private int voteCount = 0;
    private int id = 0;
    
    /**
    Create an answer with the specified title

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public Answer( String title, 
                   int id ) {
        this.title = title;
        this.id = id;
    }
    
    /**
    Set the vote count

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public void setVoteCount( int voteCount ) {
        this.voteCount = voteCount;
    }
    
    /**
    Get the vote count.

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public int getVoteCount() {
        return this.voteCount;
    }

    /**
    Get the title for this Answer... normally something like: "I really like peanuts"

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public String getTitle() {
        return this.title;
    }
    
    /**
    Get the ID for this Answer.

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public int getID() {
        return this.id;
    }
    
    /**
    Vote for this Answer, incrementing it's count by 1.

    @author <A HREF="mailto:burton@apache.org">Kevin A. Burton</A>
    @version $Id: Answer.java,v 1.3 2001/03/07 07:12:36 taylor Exp $ 
    */
    public void castVote() {
        ++this.voteCount;
    }
    
}

