/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.services.portletcache;

import org.apache.turbine.services.Service;

/**
 * <P>This service provides caching facilities for objects within the Jetspeed
 * application</p>
 * 
 * <strong>This interface is designed for backward compability with the
 * previous memory cache interfaces. It may be deprecated once the caching
 * API is refactored</strong>
 *
 * @author <a href="mailto:burton@apache.org">Kevin A. Burton</a>
 * @author <a href="mailto:raphael@apache.org">Raphal Luta</a>
 * @version $Id: PortletCacheService.java,v 1.4 2004/02/23 03:34:54 jford Exp $
 */
public interface PortletCacheService extends Service {

    public String SERVICE_NAME = "PortletCache";
        
    /**
     * Add a Cacheable object to the cache.
     *
     * @param item the object to store in the Cache
     */
    public void addCacheable( Cacheable item );
    
    /**
     * Removes a Cacheable object from the cache based on its handle
     *
     * @param handle the identifier of the object to remove
     */
    public void removeCacheable( String handle );
    
    /**
     * Retrieves a Cacheable object from the cache.
     *
     * @param handle the identifier of the object we wish to retrieve
     * @return the cacehd object or null if not found
     */
    public Cacheable getCacheable( String handle );

}

